function M= readMTLmetadata(fn)

% Read text file
fid = fopen(fn);
if(fid <= 0)
    disp(['Cannot open file: ' fn]);
end
A = textscan(fid, '%s%s', 'Delimiter', '=', 'MultipleDelimsAsOne', true, 'BufSize', 65536);
fclose(fid);

% Split file based on =
K = A{1};
V = A{2};

% Remove spaces
K = cellfun(@strtrim, K, 'UniformOutput', false);
V = cellfun(@strtrim, V, 'UniformOutput', false);
N = str2double(V);

% Build a struct, throwing away nestedness of the MTL file and relying on
% data having it's own name
R = struct();
for i = 2:length(V)
    if(isnan(N(i)))
        v = strrep(V{i}, '"', '');
    else
        v = N(i);
    end 
    R.(K{i}) =  v;
end


% Build a Structure with the data we care about
M = struct();

M.METAFORMAT              = 'MTL';


M.SPACECRAFT_ID           = R.SPACECRAFT_ID;
M.SPACECRAFT_ID           = str2num(M.SPACECRAFT_ID(end));

M.SUN_ELEVATION           = R.SUN_ELEVATION;
M.SUN_AZIMUTH             = R.SUN_AZIMUTH;
    
% Old or New MTL format?
if isfield(R, 'LMAX_BAND1')
% Get data from OLD MTL format 
    M.UTM_ZONE                = R.ZONE_NUMBER;
    M.ACQUISITION_DATE        = R.ACQUISITION_DATE;
    M.WRS_PATH                = R.WRS_PATH;
    M.WRS_ROW                 = R.STARTING_ROW;
    
    M.PRODUCT_LL_CORNER_MAPX  = R.PRODUCT_LL_CORNER_MAPX;
    M.PRODUCT_LL_CORNER_MAPY  = R.PRODUCT_LL_CORNER_MAPY;
    M.PRODUCT_UR_CORNER_MAPX  = R.PRODUCT_UR_CORNER_MAPX;
    M.PRODUCT_UR_CORNER_MAPY  = R.PRODUCT_UR_CORNER_MAPY;
    
    % Rarely, this is missing, so get an approximate time
    if ~isfield(R, 'SCENE_CENTER_SCAN_TIME')
        R.SCENE_CENTER_SCAN_TIME = [R.DATEHOUR_CONTACT_PERIOD(6:7) ':00:00Z'];
    end
    M.SCENE_CENTER_SCAN_TIME  = R.SCENE_CENTER_SCAN_TIME;

    
    % Old-style MTL for Landsat 7
    if M.SPACECRAFT_ID == 7
        % Use transforms for band 6_1
        V = [R.LMIN_BAND1 R.LMIN_BAND2 R.LMIN_BAND3 R.LMIN_BAND4 R.LMIN_BAND5 R.LMIN_BAND61 R.LMIN_BAND7;
             R.LMAX_BAND1 R.LMAX_BAND2 R.LMAX_BAND3 R.LMAX_BAND4 R.LMAX_BAND5 R.LMAX_BAND61 R.LMAX_BAND7];
    
        Q = [R.QCALMIN_BAND1 R.QCALMIN_BAND2 R.QCALMIN_BAND3 R.QCALMIN_BAND4 R.QCALMIN_BAND5 R.QCALMIN_BAND61 R.QCALMIN_BAND7;
             R.QCALMAX_BAND1 R.QCALMAX_BAND2 R.QCALMAX_BAND3 R.QCALMAX_BAND4 R.QCALMAX_BAND5 R.QCALMAX_BAND61 R.QCALMAX_BAND7];
         
        M.FILENAMES = {R.BAND1_FILE_NAME; R.BAND2_FILE_NAME; R.BAND3_FILE_NAME; R.BAND4_FILE_NAME; R.BAND5_FILE_NAME; R.BAND61_FILE_NAME; R.BAND7_FILE_NAME};
    else
        V = [R.LMIN_BAND1 R.LMIN_BAND2 R.LMIN_BAND3 R.LMIN_BAND4 R.LMIN_BAND5 R.LMIN_BAND6 R.LMIN_BAND7;
             R.LMAX_BAND1 R.LMAX_BAND2 R.LMAX_BAND3 R.LMAX_BAND4 R.LMAX_BAND5 R.LMAX_BAND6 R.LMAX_BAND7];

        Q = [R.QCALMIN_BAND1 R.QCALMIN_BAND2 R.QCALMIN_BAND3 R.QCALMIN_BAND4 R.QCALMIN_BAND5 R.QCALMIN_BAND6 R.QCALMIN_BAND7;
             R.QCALMAX_BAND1 R.QCALMAX_BAND2 R.QCALMAX_BAND3 R.QCALMAX_BAND4 R.QCALMAX_BAND5 R.QCALMAX_BAND6 R.QCALMAX_BAND7];

        M.FILENAMES = {R.BAND1_FILE_NAME; R.BAND2_FILE_NAME; R.BAND3_FILE_NAME; R.BAND4_FILE_NAME; R.BAND5_FILE_NAME; R.BAND6_FILE_NAME; R.BAND7_FILE_NAME};
    end
     
else
% Get data from NEW MTL format
    M.UTM_ZONE                = R.UTM_ZONE;
    M.ACQUISITION_DATE        = R.DATE_ACQUIRED;
    M.WRS_PATH                = R.WRS_PATH;
    M.WRS_ROW                 = R.WRS_ROW;
    M.SCENE_CENTER_SCAN_TIME  = R.SCENE_CENTER_TIME;

    M.PRODUCT_LL_CORNER_MAPX  = R.CORNER_LL_PROJECTION_X_PRODUCT;
    M.PRODUCT_LL_CORNER_MAPY  = R.CORNER_LL_PROJECTION_Y_PRODUCT;
    M.PRODUCT_UR_CORNER_MAPX  = R.CORNER_UR_PROJECTION_X_PRODUCT;
    M.PRODUCT_UR_CORNER_MAPY  = R.CORNER_UR_PROJECTION_Y_PRODUCT;

    switch M.SPACECRAFT_ID
    case 8
        V = [R.RADIANCE_MINIMUM_BAND_2 R.RADIANCE_MINIMUM_BAND_3 R.RADIANCE_MINIMUM_BAND_4 R.RADIANCE_MINIMUM_BAND_5 R.RADIANCE_MINIMUM_BAND_6 R.RADIANCE_MINIMUM_BAND_10 R.RADIANCE_MINIMUM_BAND_7;
             R.RADIANCE_MAXIMUM_BAND_2 R.RADIANCE_MAXIMUM_BAND_3 R.RADIANCE_MAXIMUM_BAND_4 R.RADIANCE_MAXIMUM_BAND_5 R.RADIANCE_MAXIMUM_BAND_6 R.RADIANCE_MAXIMUM_BAND_10 R.RADIANCE_MAXIMUM_BAND_7];

        Q = [R.QUANTIZE_CAL_MIN_BAND_2 R.QUANTIZE_CAL_MIN_BAND_3 R.QUANTIZE_CAL_MIN_BAND_4 R.QUANTIZE_CAL_MIN_BAND_5 R.QUANTIZE_CAL_MIN_BAND_6 R.QUANTIZE_CAL_MIN_BAND_10 R.QUANTIZE_CAL_MIN_BAND_7;
             R.QUANTIZE_CAL_MAX_BAND_2 R.QUANTIZE_CAL_MAX_BAND_3 R.QUANTIZE_CAL_MAX_BAND_4 R.QUANTIZE_CAL_MAX_BAND_5 R.QUANTIZE_CAL_MAX_BAND_6 R.QUANTIZE_CAL_MAX_BAND_10 R.QUANTIZE_CAL_MAX_BAND_7];

        M.FILENAMES = {R.FILE_NAME_BAND_2; R.FILE_NAME_BAND_3; R.FILE_NAME_BAND_4; R.FILE_NAME_BAND_5; R.FILE_NAME_BAND_6; R.FILE_NAME_BAND_10; R.FILE_NAME_BAND_7};
        M.K1 = R.K1_CONSTANT_BAND_10;
        M.K2 = R.K2_CONSTANT_BAND_10;
        
        if isfield(R, 'REFLECTANCE_MAXIMUM_BAND_1')
            M.GAIN_REF   = [R.REFLECTANCE_MULT_BAND_2 R.REFLECTANCE_MULT_BAND_3 R.REFLECTANCE_MULT_BAND_4 R.REFLECTANCE_MULT_BAND_5 R.REFLECTANCE_MULT_BAND_6 R.RADIANCE_MULT_BAND_10 R.REFLECTANCE_MULT_BAND_7];
            M.OFFSET_REF = [R.REFLECTANCE_ADD_BAND_2  R.REFLECTANCE_ADD_BAND_3  R.REFLECTANCE_ADD_BAND_4  R.REFLECTANCE_ADD_BAND_5  R.REFLECTANCE_ADD_BAND_6  R.RADIANCE_ADD_BAND_10  R.REFLECTANCE_ADD_BAND_7];
        end 
    
    case 7
        %Use transforms for band 6_1
        V = [R.RADIANCE_MINIMUM_BAND_1 R.RADIANCE_MINIMUM_BAND_2 R.RADIANCE_MINIMUM_BAND_3 R.RADIANCE_MINIMUM_BAND_4 R.RADIANCE_MINIMUM_BAND_5 R.RADIANCE_MINIMUM_BAND_6_VCID_1 R.RADIANCE_MINIMUM_BAND_7;
             R.RADIANCE_MAXIMUM_BAND_1 R.RADIANCE_MAXIMUM_BAND_2 R.RADIANCE_MAXIMUM_BAND_3 R.RADIANCE_MAXIMUM_BAND_4 R.RADIANCE_MAXIMUM_BAND_5 R.RADIANCE_MAXIMUM_BAND_6_VCID_1 R.RADIANCE_MAXIMUM_BAND_7];
    
        Q = [R.QUANTIZE_CAL_MIN_BAND_1 R.QUANTIZE_CAL_MIN_BAND_2 R.QUANTIZE_CAL_MIN_BAND_3 R.QUANTIZE_CAL_MIN_BAND_4 R.QUANTIZE_CAL_MIN_BAND_5 R.QUANTIZE_CAL_MIN_BAND_6_VCID_1 R.QUANTIZE_CAL_MIN_BAND_7;
             R.QUANTIZE_CAL_MAX_BAND_1 R.QUANTIZE_CAL_MAX_BAND_2 R.QUANTIZE_CAL_MAX_BAND_3 R.QUANTIZE_CAL_MAX_BAND_4 R.QUANTIZE_CAL_MAX_BAND_5 R.QUANTIZE_CAL_MAX_BAND_6_VCID_1 R.QUANTIZE_CAL_MAX_BAND_7];
        
        M.FILENAMES = {R.FILE_NAME_BAND_1; R.FILE_NAME_BAND_2; R.FILE_NAME_BAND_3; R.FILE_NAME_BAND_4; R.FILE_NAME_BAND_5; R.FILE_NAME_BAND_6_VCID_1; R.FILE_NAME_BAND_7};
        
         
    otherwise
        V = [R.RADIANCE_MINIMUM_BAND_1 R.RADIANCE_MINIMUM_BAND_2 R.RADIANCE_MINIMUM_BAND_3 R.RADIANCE_MINIMUM_BAND_4 R.RADIANCE_MINIMUM_BAND_5 R.RADIANCE_MINIMUM_BAND_6 R.RADIANCE_MINIMUM_BAND_7;
             R.RADIANCE_MAXIMUM_BAND_1 R.RADIANCE_MAXIMUM_BAND_2 R.RADIANCE_MAXIMUM_BAND_3 R.RADIANCE_MAXIMUM_BAND_4 R.RADIANCE_MAXIMUM_BAND_5 R.RADIANCE_MAXIMUM_BAND_6 R.RADIANCE_MAXIMUM_BAND_7];

        Q = [R.QUANTIZE_CAL_MIN_BAND_1 R.QUANTIZE_CAL_MIN_BAND_2 R.QUANTIZE_CAL_MIN_BAND_3 R.QUANTIZE_CAL_MIN_BAND_4 R.QUANTIZE_CAL_MIN_BAND_5 R.QUANTIZE_CAL_MIN_BAND_6 R.QUANTIZE_CAL_MIN_BAND_7;
             R.QUANTIZE_CAL_MAX_BAND_1 R.QUANTIZE_CAL_MAX_BAND_2 R.QUANTIZE_CAL_MAX_BAND_3 R.QUANTIZE_CAL_MAX_BAND_4 R.QUANTIZE_CAL_MAX_BAND_5 R.QUANTIZE_CAL_MAX_BAND_6 R.QUANTIZE_CAL_MAX_BAND_7];
    
        M.FILENAMES = {R.FILE_NAME_BAND_1; R.FILE_NAME_BAND_2; R.FILE_NAME_BAND_3; R.FILE_NAME_BAND_4; R.FILE_NAME_BAND_5; R.FILE_NAME_BAND_6; R.FILE_NAME_BAND_7};
    end
end

M.EXTENT = [M.PRODUCT_LL_CORNER_MAPY M.PRODUCT_LL_CORNER_MAPX; M.PRODUCT_UR_CORNER_MAPY M.PRODUCT_UR_CORNER_MAPX]; 
M.GAIN   = (V(2,:)-V(1,:)) ./ (Q(2,:)-Q(1,:));
M.OFFSET = V(1,:) - M.GAIN;    



